/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: updateaccessor.hxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: hr $ $Date: 2005/12/28 17:31:29 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef CONFIGMGR_UPDATEACCESSOR_HXX
#define CONFIGMGR_UPDATEACCESSOR_HXX

#ifndef CONFIGMGR_UTILITY_HXX_
#include "utility.hxx"
#endif
#ifndef CONFIGMGR_MEMORYMODEL_HXX
#include "memorymodel.hxx"
#endif
#ifndef CONFIGMGR_ACCESSOR_HXX
#include "accessor.hxx"
#endif

namespace configmgr
{
// -----------------------------------------------------------------------------
    namespace memory
    {
    // -------------------------------------------------------------------------
        class Segment;
        class SegmentHeap;
    // -------------------------------------------------------------------------

        class Allocator
        {
            SegmentHeap *   m_heap;

        public:
            typedef memory::Address Address;
            typedef memory::HeapSize Size;

            Address allocate(Size _sz) const;
            void deallocate(Address _addr) const;

            void *      access(Address _addr) const;
            Address     address(void const * _p) const;

        private:
            friend class UpdateAccessor;

            Allocator(SegmentHeap * _heap) : m_heap(_heap) {}
            char * heap_base() const;
        };
    // -------------------------------------------------------------------------
        class UpdateAccessor : Noncopyable
        {
            Segment *       m_segment;
            SegmentHeap *   m_heap;
        public:
            typedef Accessor::AddressType AddressType;

            /// constructs an accessor on the given Segment
            explicit UpdateAccessor(Segment * segment);
            ~UpdateAccessor();

            bool is() const { return m_segment != 0; }

        // Warning: The accessor and any addresses validated by it
        //          will be valid only until the next call on this->allocator()
            Accessor accessor() const;
            Accessor downgrade(); // return accessor and clear
            void clear();

            AddressType address(void const * _p) const
            { return (this->is()) ? reduce(_p) : Pointer();}

            void * access(AddressType _p) const
            { return (this->is()) ? resolve(_p) : Accessor::failNull();}

            void * validate(AddressType _p) const
            { return (this->is() && _p.is()) ? resolve(_p) : Accessor::failNull(); }

        // allocation support
            Allocator allocator() const { return Allocator(m_heap); }
        private:
            char * heap_base() const;
            void * resolve(AddressType _p) const;
            AddressType  reduce(void const * _p) const;
        };
    // -------------------------------------------------------------------------

  #if OSL_DEBUG_LEVEL == 0
        inline
        void * UpdateAccessor::resolve(AddressType _p) const
        { return _p.is() ? heap_base() + _p.value() : NULL; }

        inline
        Pointer  UpdateAccessor::reduce(void const * _p) const
        { return _p ? Pointer(static_cast<char const *>(_p) - heap_base()) : Pointer(); }
  #endif // OSL_DEBUG_LEVEL

  #if OSL_DEBUG_LEVEL == 0
        inline
        void * Allocator::access(Address _addr) const
        { return _addr ? heap_base() + _addr : NULL; }

        inline
        Allocator::Address  Allocator::address(void const * _p) const
        { return _p ? (static_cast<char const *>(_p) - heap_base()) : 0; }
  #endif // OSL_DEBUG_LEVEL

    // -------------------------------------------------------------------------
    }
// -----------------------------------------------------------------------------
} // namespace configmgr

#endif // CONFIGMGR_UPDATEACCESSOR_HXX

